const int N = 300;
const unsigned long stopTime = 500;

void setup() {
  Serial.begin(600);

  for (int i = 8; i < 12; ++i) {
    pinMode(i, INPUT_PULLUP);
  }

  for (int i = 4; i < 8; ++i) {
    pinMode(i, OUTPUT);
  }
}

void loop() {
  int8_t currState = getState();
  unsigned long currTime = millis();
  int8_t noteArray[N];
  long intervalArray[N];

  if (int8_t prevState = getState()) {
    unsigned long prevTime = millis();
    int i = 0;
    while (currState || (currTime - prevTime) < stopTime) {
      currState = getState();
      currTime = millis();
      setState(currState);
      if (currState != prevState) {
        if (i < N - 1) {

          noteArray[i] = prevState;
          intervalArray[i] = currTime - prevTime;
          prevState = currState;
          prevTime = currTime;
          i++;
        } else {
          Serial.print("Overflow");
        }
      }
    }
    noteArray[i] = -1;
    intervalArray[i] = -1;
    sendData(noteArray, intervalArray);
    playTune(noteArray, intervalArray);
  }
  delay(10);
}

int8_t getState() {
  if (!digitalRead(11)) {
    return 1;
  }
  if (!digitalRead(10)) {
    return 2;
  }
  if (!digitalRead(9)) {
    return 3;
  }
  if (!digitalRead(8)) {
    return 4;
  }
  return 0;
}

void sendDataPrint(int8_t* noteArray, long* intervalArray) {

  Serial.print("\n");

  int i = 0;
  while (noteArray[i] != -1) {
    Serial.print(noteArray[i]);
    Serial.print(" ");
    ++i;
  }
  Serial.print("\n");

  i = 0;
  while (intervalArray[i] != -1) {
    Serial.print(intervalArray[i]);
    Serial.print(" ");
    ++i;
  }
  Serial.print("\n\n");
}

void sendData(int8_t* noteArray, long* intervalArray) {
  Serial.print("notes\n");

  int i = 0;
  while (noteArray[i] != -1) {
    Serial.write(noteArray[i]);
    ++i;
  }
  Serial.print("\n");
  i = 0;
  while (intervalArray[i] != -1) {
    char* interval = (char*)&intervalArray[i];
    for (int j = 0; j < 4; ++j) {
      Serial.write(interval[j]);
    }
    ++i;
  }
  Serial.print("\n");
}



void setState(int8_t state) {
  for (int i = 4; i < 8; ++i) {
    if (i == 8 - state) {
      digitalWrite(i, 1);
    } else {
      digitalWrite(i, 0);
    }
  }
}

void playTune(int8_t* noteArray, long* intervalArray) {
  int i = 0;
  while (noteArray[i] != -1) {
    setState(noteArray[i]);
    unsigned long startTime = millis();
    while (millis() - startTime < intervalArray[i]) {
    }
    ++i;
  }
  setState(0);
}